@@ -4,7 +4,6 @@
 
 import logging
 from pathlib import Path
-import shutil
 import textwrap
 
 import numpy as np
@@ -13,7 +12,7 @@ import seekpath
 import dftbplus_step
 import seamm
 import seamm.data
-from seamm_util import units_class
+from seamm_util import Q_, units_class
 import seamm_util.printing as printing
 from seamm_util.printing import FormattedText as __
 
@@ -40,6 +39,7 @@ class BandStructure(DftbBase):
         self.description = ["Band Structure for DFTB+"]
         self.points = None  # Points along graphs of symmetry lines
         self.labels = None  # Labels of the symmetry lines
+        self.energy_step = None  # The step that got the energy and density
 
     @property
     def header(self):
@@ -87,19 +87,10 @@ class BandStructure(DftbBase):
         self.description.append(__(self.description_text(PP), **PP, indent=self.indent))
 
         # Currently use the previous energy step as source of the density
-        energy_in = None
-        step_no = len(self.parent._steps[::-1])
-        for step in self.parent._steps[::-1]:
-            step_no -= 1
-            if isinstance(step, dftbplus_step.Energy):
-                energy_in = step.get_input()
-                break
-
-        directory = Path(self.directory)
-        to_path = directory / "charges.dat"
-        from_path = directory.parent / str(step_no) / "charges.dat"
-        print(f"copy {from_path} {to_path}")
-        shutil.copy2(from_path, to_path)
+        self.energy_step = self.get_previous_charges()
+        if self.energy_step is None:
+            raise RuntimeError("Could not find charges from previous step!")
+        energy_in = self.energy_step.get_input()
 
         H = energy_in["Hamiltonian"]
         if "DFTB" in H:
@@ -116,6 +107,7 @@ class BandStructure(DftbBase):
         result = {
             "Options": {
                 "ReadChargesAsText": "Yes",
+                "SkipChargeTest": "Yes",
             },
             "Hamiltonian": H,
         }
@@ -129,8 +121,23 @@ class BandStructure(DftbBase):
         text = "Prepared the band structure graph."
 
         # Prepare the band structure graph(s)
+        if "fermi_level" in self.energy_step.results:
+            Efermi = list(
+                Q_(self.energy_step.results["fermi_level"], "hartree")
+                .to("eV")
+                .magnitude
+            )
+        else:
+            raise RuntimeError("Serious problem in the Band Structure: no Fermi level!")
+
+        dos_path = Path(self.energy_step.directory) / "dos_total.dat"
+        if not dos_path.exists():
+            dos_path = None
+
         wd = Path(self.directory)
-        self.band_structure(wd / "band.out", self.points, self.labels)
+        self.band_structure(
+            wd / "band.out", self.points, self.labels, Efermi=Efermi, dos_path=dos_path
+        )
 
         printer.normal(__(text, **data, indent=self.indent + 4 * " "))
 
@@ -148,7 +155,7 @@ class BandStructure(DftbBase):
         cell_data = configuration.primitive_cell()
 
         # Get the lines
-        seekpath_output = seekpath.get_path(cell_data, with_time_reversal=True)
+        seekpath_output = seekpath.get_path(cell_data[0:3], with_time_reversal=True)
 
         # Get the total length of the path.
         total_length = 0.0
