@@ -162,6 +162,7 @@ class Energy(DftbBase):
 
         # Need the configuration for charges, spins, etc.
         system, configuration = self.get_system_configuration(None)
+        periodicity = configuration.periodicity
         atoms = configuration.atoms
 
         # Have to fix formatting for printing...
@@ -211,15 +212,47 @@ class Energy(DftbBase):
                 hamiltonian["MaxSCCIterations"] = P["MaxSCCIterations"]
                 hamiltonian["ShellResolvedSCC"] = P["ShellResolvedSCC"].capitalize()
 
-                if P["use atom charges"] == "yes" and "charge" in atoms:
-                    skip = False
+                have_charges = "charge" in atoms
+                if have_charges:
                     for tmp in atoms["charge"]:
                         if tmp is None:
-                            skip = True
-                    if not skip:
+                            have_charges = False
+                            break
+                initial_charges = P["initial charges"]
+                if initial_charges == "default":
+                    # Use charge file from previous step or charges on atoms
+                    if self.get_previous_charges():
+                        hamiltonian["ReadInitialCharges"] = "Yes"
+                    elif have_charges:
+                        if periodicity == 0:
+                            charges = [*atoms["charge"]]
+                        else:
+                            charges = [
+                                atoms["charge"][i] for i in self.mapping_from_primitive
+                            ]
+                        # Ensure sums exactly to charge
+                        delta = (configuration.charge - sum(charges)) / len(charges)
                         hamiltonian["InitialCharges"] = {
                             "AllAtomCharges": "{"
-                            + f"{', '.join(str(c) for c in atoms['charge'])}"
+                            + f"{', '.join(str(c + delta) for c in charges)}"
+                            + "}"
+                        }
+                elif initial_charges == "from previous step":
+                    if self.get_previous_charges():
+                        hamiltonian["ReadInitialCharges"] = "Yes"
+                elif initial_charges == "from structure":
+                    if have_charges:
+                        if periodicity == 0:
+                            charges = [*atoms["charge"]]
+                        else:
+                            charges = [
+                                atoms["charge"][i] for i in self.mapping_from_primitive
+                            ]
+                        # Ensure sums exactly to charge
+                        delta = (configuration.charge - sum(charges)) / len(charges)
+                        hamiltonian["InitialCharges"] = {
+                            "AllAtomCharges": "{"
+                            + f"{', '.join(str(c + delta) for c in charges)}"
                             + "}"
                         }
 
@@ -285,26 +318,55 @@ class Energy(DftbBase):
         hamiltonian["Charge"] = configuration.charge
         multiplicity = configuration.spin_multiplicity
 
-        if multiplicity == 1 and P["SpinPolarisation"] == "none":
+        have_spins = "spin" in atoms
+        if have_spins:
+            for tmp in atoms["spin"]:
+                if tmp is None:
+                    have_spins = False
+                    break
+
+        if P["SpinPolarisation"] == "none":
+            hamiltonian["SpinPolarisation"] = {}
+        elif (
+            periodicity == 0
+            and multiplicity == 1
+            and P["SpinPolarisation"] == "from system"
+        ):
+            hamiltonian["SpinPolarisation"] = {}
+        elif (
+            periodicity != 0
+            and P["SpinPolarisation"] == "from system"
+            and multiplicity == 1
+            and not have_spins
+        ):
             hamiltonian["SpinPolarisation"] = {}
         else:
-            noncolinear = P["SpinPolarisation"] == "noncolinear"
-
-            have_spins = "spin" in atoms
-            if have_spins:
-                for tmp in atoms["spin"]:
-                    if tmp is None:
-                        have_spins = False
+            noncollinear = P["SpinPolarisation"] == "noncollinear"
 
             H = hamiltonian["SpinPolarisation"] = {}
-            if noncolinear:
-                section = H["NonColinear"] = {}
+            if noncollinear:
+                section = H["NonCollinear"] = {}
             else:
-                section = H["Colinear"] = {}
-                if have_spins:
-                    section["InitialSpins"] = {"AllAtomSpins": [*atoms["spin"]]}
-                else:
-                    section["UnpairedElectrons"] = multiplicity - 1
+                section = H["Collinear"] = {}
+                reading_charge_file = (
+                    "ReadInitialCharges" in hamiltonian
+                    and hamiltonian["ReadInitialCharges"] == "Yes"
+                )
+                if not reading_charge_file:
+                    if have_spins:
+                        if periodicity == 0:
+                            spins = atoms["spin"]
+                        else:
+                            spins = [
+                                atoms["spin"][i] for i in self.mapping_from_primitive
+                            ]
+                        section["InitialSpins"] = {
+                            "AllAtomSpins": "{"
+                            + f"{', '.join(str(c) for c in spins)}"
+                            + "}"
+                        }
+                    else:
+                        section["UnpairedElectrons"] = multiplicity - 1
 
             section["RelaxTotalSpin"] = P["RelaxTotalSpin"].capitalize()
 
@@ -332,7 +394,7 @@ class Energy(DftbBase):
                 constants = spin_constant_data["GGA"]
 
             # Bit of a kludgy test. If not shell-resolved there is one constant
-            # per shell, i.e. 1, 2 or 3 for s, p, d. If reolved, there are 1, 4, 9.
+            # per shell, i.e. 1, 2 or 3 for s, p, d. If resolved, there are 1, 4, 9.
             shell_resolved = False
             for symbol in symbols:
                 if len(constants[symbol]) > 3:
@@ -401,11 +463,11 @@ class Energy(DftbBase):
             oc = 0.0 if nc % 2 == 1 else 0.5
             kmesh = (
                 "SupercellFolding {\n"
-                f"    {na} 0 0\n"
-                f"    0 {nb} 0\n"
-                f"    0 0 {nc}\n"
-                f"    {oa} {ob} {oc}\n"
-                "}"
+                f"            {na} 0 0\n"
+                f"            0 {nb} 0\n"
+                f"            0 0 {nc}\n"
+                f"            {oa} {ob} {oc}\n"
+                "        }"
             )
             hamiltonian["KPointsAndWeights"] = kmesh
             self.description.append(
@@ -416,6 +478,20 @@ class Energy(DftbBase):
                 )
             )
 
+        # If reading the charge file, use a text file
+        if (
+            "ReadInitialCharges" in hamiltonian
+            and hamiltonian["ReadInitialCharges"] == "Yes"
+        ):
+            if "Options" not in result:
+                result["Options"] = {
+                    "ReadChargesAsText": "Yes",
+                    "SkipChargeTest": "Yes",
+                }
+            else:
+                result["Options"]["ReadChargesAsText"] = "Yes"
+                result["Options"]["SkipChargeTest"] = "Yes"
+
         return result
 
     def analyze(self, indent="", data={}, out=[]):
@@ -451,21 +527,31 @@ class Energy(DftbBase):
             data["energy of formation"] = None
 
         # Prepare the DOS graph(s)
+        if "fermi_level" in data:
+            Efermi = list(Q_(data["fermi_level"], "hartree").to("eV").magnitude)
+        else:
+            Efermi = [0.0]
         wd = Path(self.directory)
-        self.dos(wd / "band.out")
+        self.dos(wd / "band.out", Efermi=Efermi)
 
         text_lines = []
         # Get charges and spins, etc.
         system, configuration = self.get_system_configuration(None)
         symbols = configuration.atoms.symbols
         atoms = configuration.atoms
+        periodicity = configuration.periodicity
         if "gross_atomic_charges" in data:
             # Add to atoms (in coordinate table)
             if "charge" not in atoms:
                 atoms.add_attribute(
                     "charge", coltype="float", configuration_dependent=True
                 )
-            atoms["charge"][0:] = data["gross_atomic_charges"]
+            charges = data["gross_atomic_charges"]
+            if periodicity == 0:
+                atoms["charge"][0:] = charges
+            else:
+                tmp = [charges[i] for i in self.mapping_to_primitive]
+                atoms["charge"][0:] = tmp
 
             # Print the charges and dump to a csv file
             table = {
@@ -520,8 +606,12 @@ class Energy(DftbBase):
                 atoms.add_attribute(
                     "spin", coltype="float", configuration_dependent=True
                 )
-            atoms["spin"][0:] = data["gross_atomic_spins"][0]
-
+            spins = data["gross_atomic_spins"][0]
+            if periodicity == 0:
+                atoms["spin"][0:] = spins
+            else:
+                tmp = [spins[i] for i in self.mapping_to_primitive]
+                atoms["spin"][0:] = tmp
         text = str(__(text, **data, indent=self.indent + 4 * " "))
         text += "\n\n"
         text += textwrap.indent("\n".join(text_lines), self.indent + 7 * " ")
